#!/bin/bash
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


# build.sh generates a Super Grub Disk filesystem
# usage devuelve la sintaxis correcta a la hora de ejecutar
# el build.sh que es el creador de grub

#set -x
#set -v

usage()
{
        cat <<EOF
Usage: $0 [options] floppy
Creates Super Grub Disk floppy on /tmp/sdg/.

Usage: $0 [options] cdrom
Creates Super Grub Super cdrom on /tmp/sdg/.

options:
        -h, --help      print this message and exit

EOF
        exit 1
}

create_all_menus()
{
	# Read configuration if it exists
	if [ -d "$CONFIG_FOLDER" ] ;   then
		for i in $CONFIG_FOLDER/*.sh
		do
			source $i
		done
	fi
	# Create menu
	create_menu_lst > "${SDG_MENU_LST[LEVEL]}"
	# Create help
	#create_help_lst > "${SDG_HELP_LST[LEVEL]}"
	# Substitutions
	substitute_all_files
	
} # End of create_all_menus

# Seek folders
# help us echoing the 
# various grub entries that are in
# current folder
# It is an auxiliary function of create_menu_lst
seek_folders()
{
local DIR;
# Lets seek all the files in current folder
for DIR in *
do
# We want only the folders
	if [ -d "$DIR" ] ; then
# We want to process only folders with build.sh on them
		if [ -e "$DIR"/"${SDG_MODULE_ID[LEVEL]}" ] ; then
		  if [ -e "$DIR"/"${SDG_LANG_ID}" ] ; then
			SGD_CURRENT_LANG_DIR="$DIR";
		  fi
		  cd "$DIR"; levelplus;
			CURRENT_FOLDER[LEVEL]="${CURRENT_FOLDER[LEVEL - 1]}""/""$DIR"
# We update the current folder and echo the entry of this folder
			create_all_menus
# If SDG_IS_EDU variable is set to YES
# each time an option is choosed we see its help
# If it's set otherwise we see the menu directly
# Let's override this with the environment SGD_VERBOSE variable


			if [ "$SDG_IS_EDU" == "YES" ] ; then
				add_menu_entry "${SDG_TITLE[LEVEL]}" "${CURRENT_FOLDER[LEVEL]}""/""${SDG_MENU_LST[LEVEL]}" "catis" "\$(SGD_VERBOSE)"
			else
				add_menu_entry "${SDG_TITLE[LEVEL]}" "${CURRENT_FOLDER[LEVEL]}""/""${SDG_MENU_LST[LEVEL]}" "catis" "off"
			fi
			
# We go one folder down. We go one level down.
		  cd .. ; levelless;
		fi
	fi
done

} # End of seek_folders


# This function is used for creating a menu.lst
# it looks every posible folder that has its options
# and adds it to the menu

create_menu_lst()
{

# Let's put the beggining. The head
# If it exists, of course
# 
if [ -e "${SDG_MENU_HEAD[LEVEL]}" ] ; then
	cat "${SDG_MENU_HEAD[LEVEL]}"
fi

# HELP GOES HERE - BEGIN
local FILE
# If there's a help file we're going to show this one
if [ -e "${SDG_HELP_TXT[LEVEL]}" ] ; then
	add_help_cat "${CURRENT_FOLDER[LEVEL]}""/""${SDG_HELP_TXT[LEVEL]}"
# If there's more than one let's show all them
elif [ -e "${SDG_HELP_DIR[LEVEL]}" ] ; then
	cd "${SDG_HELP_DIR[LEVEL]}" 
	for FILE in *.txt
	do
		add_help_cat "${CURRENT_FOLDER[LEVEL]}""/""${SDG_HELP_DIR[LEVEL]}""/""$FILE"
	done
	cd ..
fi

# HELP GOES HERE - END


# Each menu has an entry with the previous menu

add_menu_entry "${SDG_PRE_BACK[LEVEL]}${SDG_TITLE[LEVEL - 1]}${SDG_POST_BACK[LEVEL]}" "${CURRENT_FOLDER[LEVEL - 1]}""/""${SDG_MENU_LST[LEVEL - 1]}" "catis" "off"

# And each menu has an entry of its own name with the help

add_menu_entry "${SDG_PRE_CURRENT[LEVEL]}${SDG_TITLE[LEVEL]}${SDG_POST_CURRENT[LEVEL]}" "${CURRENT_FOLDER[LEVEL]}""/""${SDG_MENU_LST[LEVEL]}" "catis" "\$(SGD_VERBOSE)"

# Then we should look among folders for scripts

if [ -e "${SDG_MENU_BODY[LEVEL]}" ] ; then
	cat "${SDG_MENU_BODY[LEVEL]}"
else
seek_folders
fi

# Let's put the end. The tail.
# If it exists, of course
# 
if [ -e "${SDG_MENU_TAIL[LEVEL]}" ] ; then
	cat "${SDG_MENU_TAIL[LEVEL]}"
fi



} # End of create_menu_lst


# Create a device
# makes a generic device filesystem
# that we will use for a template
# to build devices
create_device()
{
read_sgd_version
# Update SGD edition
#======================
SGD_EDITION=`head -n 1 edition.txt`

# We declare the folder where configuration is going to be found
SDG_TRANSLATE_SH="base_dir.sh"
CONFIG_FOLDER="conf"
# We need to know how to identify a module or language folder
SDG_MODULE_ID[LEVEL]="sdg_module"
SDG_COMMON_DIR="../common"
SDG_TRANSLATE_BASE_DIR_0="../lng"
SDG_TRANSLATE_BASE_DIR_1="S10en"
# SDG_TARGET is the folder where
# the SDG is going to be created
SDG_TARGET="$PROGRAM_TARGET"


# Prepare target folder

if [ -d $SDG_TARGET ] ; then
  # If target folder exists we should delete it and create it again.
  rm -R $SDG_TARGET
  mkdir -p $SDG_TARGET
else
  # If target folder does exit we create it.
  mkdir -p $SDG_TARGET;
fi


# We copy the lang folders
# in order to make translations easier


local DIR;
# Lets seek all the files in current folder
# Explanation of languages copies
# Catalan, cat, is a language which is spoken in Spain.
# In Spain we speak Spanish
# So if a Catalan translator does not want to translate everything
# he can translate a part and he will have the rest translated into Spanish
# 
# because he has decided that in cat/conf/translate.sh 
# SDG_TRANSLATE_BASE_DIR variable is set to es
# 
# If the spanish translator (es) is also lazy he will have some explanations
# in English (which is the SDG_TRANSLATE_BASE_DIR_1 variable)
#
# And finally we have the "european bash language" which is in some sort
# written in the lng dir (SDG_TRANSLATE_BASE_DIR_0 variable) 

for i in $CONFIG_FOLDER/*.sh
		do
			source $i
		done

SDG_MODULE_ID="sdg_module"
SDG_LANG_ID="lang_module"

# First fetch interesting things from dev_sgd

cp * $SDG_TARGET
cp -R gpl $SDG_TARGET # Useful for gpl license
cp -R brs $SDG_TARGET # Useful for boot records for FIXMBR command

cd tr

# Second let's see the languages

for DIR in *
do
# We want only the folders
	if [ -d "$DIR" ] ; then
# We want to process only folders with sdg_module on them
		if [ -e "$DIR"/"${SDG_MODULE_ID[LEVEL]}" ] ; then
		source "$DIR"/"$CONFIG_FOLDER"/"$SDG_TRANSLATE_SH"
# Copy common algorithm folder
		cp -R "$SDG_COMMON_DIR" "$SDG_TARGET""/""$DIR"
# Copy common language folder
		cp -R "$SDG_TRANSLATE_BASE_DIR_0""/"* "$SDG_TARGET""/""$DIR"
# Copy imperium language folder (very used language)
		cp -R "$SDG_TRANSLATE_BASE_DIR_1""/"* "$SDG_TARGET""/""$DIR"
		
# Copy based-language folder
		cp -R "$SDG_TRANSLATE_BASE_DIR""/"* "$SDG_TARGET""/""$DIR"
		
# Copy the language folder
		cp -R "$DIR""/"* "$SDG_TARGET""/""$DIR"
# Set the SGD version number
#==============================
sed_string="";
sed_string="${sed_string}"`sed_partial_string  "MYSGDVERSION" "${SGD_VERSION}"`
echo "$SDG_TARGET""/""$DIR"/"${SDG_HELP_DIR[LEVEL]}""/version.txt" | xargs sed "$sed_string" -i
# Set the SGD edition
#==============================
sed_string="";
sed_string="${sed_string}"`sed_partial_string  "MYSGDEDITION" "${SGD_EDITION}"`
echo "$SDG_TARGET""/""$DIR"/"${SDG_HELP_DIR[LEVEL]}""/version.txt" | xargs sed "$sed_string" -i


		else # If it is not a sdg_module but a lang module one we want to copy it so that we have inheritance between languages
		  if [ -e "$DIR"/"$SDG_LANG_ID" ] ; then
		    cp -R "$DIR" "$SDG_TARGET"
		  fi
		fi
	fi
done


cd $SDG_TARGET

#
#
create_all_menus


} # End of create_device


# It creates a floppy filesystem
# with grub super disk on it


create_floppy()
{
#
# Crear dispositivo
create_device
#
echo "Building floppy"
# Defino variables del disquete
# El dispositivo de SDG va a ser el disquete
# 
SDG_DEVICE="(fd0)"
# substitute in every file the SDG_DEVICE
# I cannot use my substitute function, I use sed directly

#find . -type f | xargs sed -i "s/SDG_DEVICE/$SDG_DEVICE/g"
#Moved to S010_main_functions.sh
# No queremos imagenes
SDG_IMAGES=no
# Aqui habra un if de DEBUG
#
}


# It creates a cdrom filesystem
# with grub super disk on it

create_cdrom()
{
#
# Ejecutar dispositivo
create_device
#
echo "Building cdrom"
# Defino variables del cdrrom
# El dispositivo de SDG va a ser el cdrom
# 
SDG_DEVICE="(cd)"
# substitute in every file the SDG_DEVICE
#find . -type f | xargs sed -i "s/SDG_DEVICE/$SDG_DEVICE/g"
#Moved to S010_main_functions.sh
# Queremos imagenes
SDG_IMAGES=yes
# Aqui habra un if de DEBUG
#
}

create_generic()
{
# Fetch interesting path variables
source ./sgrub_configure.sh
#
# Ejecutar dispositivo
#SDG_DEVICE="\$(grub_device)" NOT USED
create_device
#
echo "MESSAGE: Building generic SGD"
# Defino variables del cdrrom
# El dispositivo de SDG va a ser el cdrom
# 
# substitute in every file the SDG_DEVICE
#find . -type f | xargs sed -i "s/SDG_DEVICE/$SDG_DEVICE/g"
#Moved to S010_main_functions.sh
# Queremos imagenes
SDG_IMAGES=yes
# Aqui habra un if de DEBUG
#
}


#Si no hay $1 saca el mensaje de error

#Programa principal
# test we have the necessary files first
#checkfiles

case "$1" in
        -h | --help)
                usage
                ;;
	floppy)
		create_generic
		#create_floppy
		;;
	cdrom)
		create_generic
		#create_cdrom
		;;

        *)
		usage
                ;;
esac

exit 0
